#pragma once

/* Genode includes */
#include <session/session.h>
#include <base/rpc.h>
#include <dataspace/capability.h>

namespace Sign
{
	using Genode::size_t;

	enum {
		SIGNATURE_SIZE   = 1024,   // maximal size in bytes
		CERTIFICATE_SIZE = 8192    // maximal size in bytes
	};

	struct Session;

	class Buffer_too_small : public Genode::Exception { };
}


struct Sign::Session : Genode::Session
{
	/**
	 * \noapi
	 */
	static const char *service_name() { return "Sign"; }

	/*
	 * A sign session consumes a dataspace capability for the server's
	 * session-object allocation, its session capability, and a dataspace
	 * capability for the communication buffer.
	 */
	enum { CAP_QUOTA = 3 };

	/**
	 * Signs a piece of data, most probably a hash.
	 */
	virtual size_t sign(void *data_buffer, size_t data_bytes, size_t buffer_size) = 0;

	/**
	  * Get the certificate used to calculate the signature.
	  */
	virtual size_t get_certificate(void *data_buffer, size_t buffer_size) = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_sign, size_t, _sign, GENODE_TYPE_LIST(Buffer_too_small), size_t);
	GENODE_RPC_THROW(Rpc_get_certificate, size_t, _get_certificate, GENODE_TYPE_LIST(Buffer_too_small), size_t);
	GENODE_RPC(Rpc_dataspace, Genode::Dataspace_capability, _dataspace);

	GENODE_RPC_INTERFACE(Rpc_sign, Rpc_get_certificate, Rpc_dataspace);
};

